buildscript {
  repositories {
    mavenLocal()
    mavenCentral()
  }

  dependencies {
    classpath 'org.yaml:snakeyaml:2.0'
  }
}

import org.yaml.snakeyaml.Yaml


import org.gradle.api.Project


import org.gradle.api.artifacts.repositories.AuthenticationContainer
import org.gradle.api.artifacts.repositories.MavenArtifactRepository
import org.gradle.api.artifacts.repositories.PasswordCredentials
import org.gradle.api.artifacts.repositories.RepositoryContentDescriptor
import org.gradle.api.logging.Logger
import org.gradle.internal.authentication.DefaultAwsImAuthentication
import org.yaml.snakeyaml.Yaml

import java.util.regex.Matcher
import java.util.regex.Pattern

interface IConfiguration {
    boolean isAutoRestrictArtifactApiGroup()

    String getIncludeArtifactApiGroupRegexp()

    List<String> getIncludeArtifactApiGroups()

    List<String> getExcludeArtifactApiGroups()
}

class MavenConfigurer {
    private Boolean isInPrivateVpc
    private final Map<String, String> aionConfig
    private final Logger logger
    private boolean credentialsMessageDisplayed = false

    MavenConfigurer(final Map<String, String> aionConfig, final Logger logger) {
        this.aionConfig = aionConfig
        this.logger = logger
    }

    MavenConfigurer(final String homeDir, final Logger logger) throws IOException {
        this.aionConfig = new HashMap<>()
        this.logger = logger

        def configFiles = new ArrayList<File>()
        def currentDir = new File(homeDir)
        while (currentDir != null) {
            def configFile = new File(currentDir, "aion.config")
            configFiles.add(configFile)
            currentDir = currentDir.getParentFile() // jump to parent directory
        }
        Collections.reverse(configFiles)

        loadConfigFiles(configFiles)
    }

    private void loadConfigFiles(final List<File> configFiles) {
        for (final File configFile : configFiles) {
            if (configFile.exists()) {
                FileInputStream is = null
                try {
                    is = new FileInputStream(configFile)
                    logInfo("tool-gradle-config loading configuration from " + configFile.getPath())
                    aionConfig.putAll(new Yaml().load(is))
                } catch (final IOException err) {
                    if (is != null) {
                        is.close()
                    }
                    throw err
                }
            }
        }
        aionConfig.putAll(System.getenv())
    }

    private void logInfo(final String message) {
        logInfo(message, null)
    }

    private void logInfo(final String message, final Exception exception) {
        if (logger == null) {
            System.out.println(message)
        } else {
            logger.info(message, exception)
        }
    }

    private boolean _hasCredentials() {
        if (!credentialsMessageDisplayed) {
            credentialsMessageDisplayed = true
            if (aionConfig.containsKey("AION_ARTIFACTS_API_AUTH_TOKEN")) {
                logInfo('Credentials (AION_ARTIFACTS_API_AUTH_TOKEN) found, using the public artifacts API endpoint.')
            } else {
                logInfo('No credentials (AION_ARTIFACTS_API_AUTH_TOKEN) found, using the private artifacts API endpoint. If you are attempting to use the public endpoint, ensure you set AION_ARTIFACTS_API_AUTH_TOKEN set in ~/aion.config.')
            }
        }
        return aionConfig.containsKey("AION_ARTIFACTS_API_AUTH_TOKEN")
    }

    String getArtifactsEndpoint() throws MalformedURLException {
        return _hasCredentials() ? aionConfig.get("AION_MAVEN_REPO_PATH_PUBLIC") : aionConfig.get("AION_MAVEN_REPO_PATH_VPC")
    }

    private boolean artifactsEndpointNeedsAuthorization() throws MalformedURLException {
        return _hasCredentials()
    }

    private String getAuthToken() {
        if (!aionConfig.containsKey("AION_ARTIFACTS_API_AUTH_TOKEN")) {
            throw new RuntimeException("AION_ARTIFACTS_API_AUTH_TOKEN (auth token) configuration is required. This can be an env var or set in ~/aion.config. DO NOT set in repo/aion.config.")
        }

        return aionConfig.get("AION_ARTIFACTS_API_AUTH_TOKEN")
    }

    void configure(final IConfiguration configuration, final MavenArtifactRepository mavenArtifactRepository) throws MalformedURLException {
        configure(configuration, null, mavenArtifactRepository)
    }

    void configure(final IConfiguration configuration, final Project project, final MavenArtifactRepository mavenArtifactRepository) throws MalformedURLException {
        final String mavenPath = getArtifactsEndpoint()

        System.out.println("MavenConfigurator.configure: ");
        System.out.println("  aionConfig.get(\"AION_ARTIFACTS_API_AUTH_TOKEN\"): " + aionConfig.get("AION_ARTIFACTS_API_AUTH_TOKEN"));
        System.out.println("  aionConfig.get(\"AION_MAVEN_REPO_PATH_PUBLIC\"): " + aionConfig.get("AION_MAVEN_REPO_PATH_PUBLIC"));
        System.out.println("  aionConfig.get(\"AION_MAVEN_REPO_PATH_VPC\"): " + aionConfig.get("AION_MAVEN_REPO_PATH_VPC"));
        System.out.println("  mavenPath = " + mavenPath);

        mavenArtifactRepository.setUrl(mavenPath)
        if (mavenPath.startsWith("s3://")) {
            mavenArtifactRepository.authentication(this.&configureCredentials)
            if (configuration != null) {
                mavenArtifactRepository.content({ repositoryContentDescriptor -> configureRepositoryContent(configuration, project, repositoryContentDescriptor) })
            }
        } else if (mavenPath.startsWith("https://")) {
            if (artifactsEndpointNeedsAuthorization()) {
                mavenArtifactRepository.credentials(this.&configureCredentials)
            }
            if (configuration != null) {
                mavenArtifactRepository.content({ repositoryContentDescriptor -> configureRepositoryContent(configuration, project, repositoryContentDescriptor) })
            }
        }
    }

    private void configureRepositoryContent(final IConfiguration configuration, final Project project, final RepositoryContentDescriptor repositoryContentDescriptor) {
        final def log = project == null ? { it -> System.out.println(it) } : project.getLogger().&info

        final def cb = {
            if (configuration.isAutoRestrictArtifactApiGroup()) {
                final Matcher groupPrefix = Pattern.compile("^([^.]+\\.[^.]+)").matcher(project.getGroup().toString())
                if (groupPrefix.find()) {
                    final String groupPrefixWithEscapedSeparators = groupPrefix.group(1).replaceAll("\\.", "\\\\.")
                    final String groupPattern = "${groupPrefixWithEscapedSeparators}(\\..*|\$)"
                    log("Auto include group pattern in Maven config: ${groupPattern}")
                    repositoryContentDescriptor.includeGroupByRegex(groupPattern)
                }
            }
            if (configuration.getIncludeArtifactApiGroupRegexp() != null) {
                log("Include group pattern in Maven config: " + configuration.getIncludeArtifactApiGroupRegexp())
                repositoryContentDescriptor.includeGroupByRegex(configuration.getIncludeArtifactApiGroupRegexp())
            }
            if (configuration.getIncludeArtifactApiGroups() != null) {
                configuration.getIncludeArtifactApiGroups().forEach({ group ->
                    log("Include group in Maven config: " + group)
                    repositoryContentDescriptor.includeGroup(group)
                })
            }
            if (configuration.getExcludeArtifactApiGroups() != null) {
                configuration.getIncludeArtifactApiGroups().forEach({ group ->
                    log("Exclude group in Maven config: " + group)
                    repositoryContentDescriptor.excludeGroup(group)
                })
            }
            return null
        }
        if (project != null) {
            project.afterEvaluate(cb)
        } else {
            cb()
        }
    }

    @Override
    boolean equals(Object o) {
        if (this.is(o)) return true
        if (!(o instanceof MavenConfigurer)) return false
        MavenConfigurer that = (MavenConfigurer) o
        return Objects.equals(isInPrivateVpc, that.isInPrivateVpc) &&
                Objects.equals(aionConfig, that.aionConfig) &&
                Objects.equals(logger, that.logger)
    }

    @Override
    int hashCode() {
        return Objects.hash(isInPrivateVpc, aionConfig, logger)
    }

    private void configureCredentials(final AuthenticationContainer authenticationContainer) {
        authenticationContainer.add(new DefaultAwsImAuthentication(""))
    }

    private void configureCredentials(final PasswordCredentials passwordCredentials) {
        passwordCredentials.setUsername("")
        passwordCredentials.setPassword(getAuthToken())
    }

    HashMap<String, String> getAionConfig() {
        return new HashMap<>(aionConfig);
    }
}

class ToolGradleConfigConfigurationPlugin implements Plugin<Settings> {
  static DEFAULT_TOOL_GRADLE_CONFIG_VERSION = "0.0.262-245"

  @Override
  void apply(final Settings settings) {
    configureToolVersion(settings)
  }

  void configureToolVersion(settings) {
    String toolGradleConfigVersion = null

    try {
      def dependencies = new Yaml().load(new File("dependencies.yaml").newInputStream())

      try {
        toolGradleConfigVersion = dependencies.dependencies.values().stream()
          .flatMap({ v -> v.entrySet().stream() })
          .filter({ e -> (e.key == 'tool-gradle-config' && e.value.version != null && e.value.version != "*") })
          .map({ e -> e.value.version })
          .findAny().orElse(null)
      }
      catch (err) {
      }

      if (toolGradleConfigVersion == null) {
        try {
          toolGradleConfigVersion = dependencies.lockedVersions["tool-gradle-config"]
        }
        catch (err) {
        }
      }

      if (toolGradleConfigVersion == null) {
        try {
          dependencies.values().stream()
                  .flatMap({ v -> v.values().stream() })
                  .flatMap({ v -> v.values().stream() })
                  .flatMap({ v -> v instanceof Map ? v.values().stream() : new HashMap().values().stream() })
                  .flatMap({ v -> v.stream() })
                  .filter({ v -> v instanceof String && v.startsWith('com.mastercard.tool-gradle-config') })
                  .forEach({ pkg ->
                    toolGradleConfigVersion = pkg.split(':')[2]
                    println "Using tool-gradle-config version: $toolGradleConfigVersion"
                  })
        }
        catch (err) {
        }
      }
    } catch (err) {
    }

    if (!toolGradleConfigVersion) {
      println "Using default tool-gradle-config version: $DEFAULT_TOOL_GRADLE_CONFIG_VERSION"
      toolGradleConfigVersion = DEFAULT_TOOL_GRADLE_CONFIG_VERSION
    }

    def settingsConfig = [
      isAutoRestrictArtifactApiGroup: { false },
      getIncludeArtifactApiGroups: { [] },
      getIncludeArtifactApiGroupRegexp: { "com\\.mastercard(\\..*|\$)" },
      getExcludeArtifactApiGroups: { [] },
    ] as IConfiguration
    settings.pluginManagement {
      repositories {
        mavenLocal()
        maven() { maven ->
          new MavenConfigurer(settings.rootDir.getAbsolutePath(), null).configure(settingsConfig, maven)
        }
        mavenCentral()
        gradlePluginPortal()
      }
    }

    settings.gradle.ext.initRepositories = { repositories ->
      repositories.mavenLocal()
      repositories.maven { it ->
        new MavenConfigurer(settings.rootDir.getAbsolutePath(), null).configure(settingsConfig, it)
      }

      repositories.mavenCentral()
      repositories.gradlePluginPortal()
    }

    settings.gradle.ext.initPluginRepositories = { buildscript ->
      settings.gradle.ext.initRepositories(buildscript.repositories)

      buildscript.dependencies.add('classpath', "com.mastercard.tool-gradle-config:com.mastercard.tool-gradle-config.gradle.plugin:$toolGradleConfigVersion")
      buildscript.dependencies.add('classpath', "com.mastercard.tool-gradle-config-java:com.mastercard.tool-gradle-config-java.gradle.plugin:$toolGradleConfigVersion")
      buildscript.dependencies.add('classpath', "com.mastercard.tool-gradle-config-typescript:com.mastercard.tool-gradle-config-typescript.gradle.plugin:$toolGradleConfigVersion")
    }
  }
}

apply plugin: ToolGradleConfigConfigurationPlugin
